package pl.primesoft.aztecreaderdemo.Scanning.CameraManager;

import android.content.Context;
import android.hardware.Camera;
import android.os.Build;
import android.view.Surface;
import android.view.View;
import android.view.WindowManager;

import pl.primesoft.aztecreaderdemo.Scanning.CameraManager.Utils.AutoFocusUtils;
import pl.primesoft.aztecreaderdemo.Scanning.CameraManager.Utils.FlashUtils;
import pl.primesoft.aztecreaderdemo.Scanning.CameraManager.Utils.FpsRangeUtils;
import pl.primesoft.aztecreaderdemo.Scanning.CameraManager.Utils.ZoomUtils;

/**
 * Created by wygralak on 2016-03-11.
 */
public class CameraManager {
    public static int REFOCUSING_DELAY = 2500;
    private final AutoFocusUtils autoFocusUtils;
    private ZoomUtils zoomUtils;

    private Context context;
    private Camera camera;

    public enum AutoFocusMode {
        CONTINUOUS_MODE,
        TIMER_MODE,
        ON_CLICK,
        DISABLED
    }

    public enum FpsRangeMode {
        MAXIMUM,
        MINIMUM,
        DEFAULT
    }

    public CameraManager(Context context) {
        this.context = context;
        autoFocusUtils = new AutoFocusUtils();
        zoomUtils = new ZoomUtils();
    }

    public void setupCamera() throws CameraException {
        int cameraId = getBackFacingCameraId();
        camera = Camera.open(cameraId);
        if (camera != null) {
            setCameraDisplayOrientation(cameraId);
        } else {
            throw new CameraException("Can not open camera!");
        }
    }

    private void setCameraDisplayOrientation(int cameraId) {
        android.hardware.Camera.CameraInfo info =
                new android.hardware.Camera.CameraInfo();
        android.hardware.Camera.getCameraInfo(cameraId, info);

        WindowManager windowManager = (WindowManager) context.getSystemService(Context.WINDOW_SERVICE);
        int rotation = windowManager.getDefaultDisplay().getRotation();

        int degrees = 0;
        switch (rotation) {
            case Surface.ROTATION_0:
                degrees = 0;
                break;
            case Surface.ROTATION_90:
                degrees = 90;
                break;
            case Surface.ROTATION_180:
                degrees = 180;
                break;
            case Surface.ROTATION_270:
                degrees = 270;
                break;
        }
        int result;
        if (info.facing == Camera.CameraInfo.CAMERA_FACING_FRONT) {
            result = (info.orientation + degrees) % 360;
            result = (360 - result) % 360;  // compensate the mirror
        } else {  // back-facing
            result = (info.orientation - degrees + 360) % 360;
        }
        camera.setDisplayOrientation(result);
    }

    public View.OnClickListener setupAutoFocus(AutoFocusMode autoFocusMode) throws AutoFocusUtils.AutoFocusException {
        switch (autoFocusMode) {
            case CONTINUOUS_MODE: {
                return autoFocusUtils.setupContinuousAutoFocus(camera);
            }
            case TIMER_MODE: {
                autoFocusUtils.startTimerFocusing(camera);
                return null;
            }
            case ON_CLICK: {
                return autoFocusUtils.getSingleAutoFocusOnClick(camera);
            }
        }
        return null;
    }

    public void setPreviewCallback(Camera.PreviewCallback callback) {
        camera.setPreviewCallback(callback);
    }

    private int getBackFacingCameraId() throws CameraException {
        int numberOfCameras = Camera.getNumberOfCameras();
        for (int i = 0; i < numberOfCameras; i++) {
            Camera.CameraInfo info = new Camera.CameraInfo();
            Camera.getCameraInfo(i, info);
            if (info.facing == Camera.CameraInfo.CAMERA_FACING_BACK) {
                return i;
            }
        }
        throw new CameraException("Back facing camera not found!");
    }

    public boolean isFlashAvailable() {
        return FlashUtils.isFlashAvailable(camera, context);
    }

    public String toggleFlash() {
        return FlashUtils.toggleFlash(camera);
    }

    public boolean isZoomAvailable() {
        return ZoomUtils.isZoomAvailable(camera);
    }

    public View.OnClickListener setupZooming() {
        return zoomUtils.setupZoom(camera);
    }

    public Camera getCameraInstance() {
        return camera;
    }

    public void turnOffCamera() {
        if (camera != null) {
            autoFocusUtils.turnOffAutoFocus(camera);
            camera.setPreviewCallback(null);
            camera.stopPreview();
            camera.release();
        }
    }

    public boolean isFpsRangeAvailable() {
        return FpsRangeUtils.isFpsRangeAvailable(camera);
    }

    public void setFpsRangeMode(FpsRangeMode currentFpsRangeMode) {
        switch (currentFpsRangeMode) {
            case DEFAULT: {
                FpsRangeUtils.setDefaultFpsRange(camera);
                break;
            }
            case MAXIMUM: {
                FpsRangeUtils.setMaximumFpsRange(camera);
                break;
            }
            case MINIMUM: {
                FpsRangeUtils.setMinimumFpsRange(camera);
                break;
            }
        }
    }

    /*public void exposure() {
        int minExposureCompensation = camera.getParameters().getMinExposureCompensation();
        int maxExposureCompensation = camera.getParameters().getMaxExposureCompensation();
        int exposureCompensation = camera.getParameters().getExposureCompensation();
        Camera.Parameters parameters = camera.getParameters();
        int exp = exposureCompensation == 0 ? 6 : 0;
        parameters.setExposureCompensation(exp);
        camera.setParameters(parameters);
        if (maxExposureCompensation == 0) {
            int y = -1;
        }
    }*/

    public boolean isVideoStabilizationSupported() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.ICE_CREAM_SANDWICH_MR1) {
            return camera.getParameters().isVideoStabilizationSupported();
        } else {
            return false;
        }
    }

    public void setVideoStabilization(boolean enable) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.ICE_CREAM_SANDWICH_MR1) {
            Camera.Parameters parameters = camera.getParameters();
            parameters.setVideoStabilization(enable);
            camera.setParameters(parameters);
        }
    }
}
